
/*  Connections
 *  ------------
 */

/*  Connections are necessary for the layout of directly neighboured
 *  connected nodes at the same level. This situation may occur accidently,
 *  but automatical occurs if there is a self loop.
 *
 *  On self loops: 
 *  - - - - - - -      N            A self loop of N is an edge (N,N).
 *                    / ^           To layout this, we create a dummy node
 *                   /   \          D and a pseudo node P. D and P are 
 *                  D-----P         always neighboured at the same level.
 *
 *  Only N and D is layouted. The connection of the dummy node D contains the 
 *  information needed to draw P (that is not layouted), i.e.:
 *     CTARGET(D)  -> P
 *     CEDGE(D)    -> edge (D,P)      
 *     CTARGET(P)  -> D
 *     CEDGE(P)    -> edge (D,P)      
 *
 *  On connected neigboured nodes:
 *  - - - - - - - - - - - - - - - 
 *              /    |    \       or    |     \      or     /     |
 *             A<----B---->C            B----->C           C<-----B
 *
 *  Only B is layouted. The connection of B contains the information
 *  to draw A and/or C, i.e.
 *     CTARGET(B)  -> C
 *     CEDGE(B)    -> edge (B,C)
 *     CTARGET2(B) -> A           or NULL, if not necessary
 *     CEDGE2(B)   -> edge (B,A)  or NULL, if not necessary
 *
 *     CTARGET(C)  -> B
 *     CEDGE(C)    -> edge (B,C)
 *     etc.
 *
 *  We call the connection field of B a `forward connection' and the
 *  connection field of B or A a `backward connection', because its
 *  in converse of the edge direction.
 *  Note that A or C can also contain connections, e.g. on
 *               |    \     \
 *               B---->C---->D           CTARGET(B) = C, CTARGET(C) = D, ...
 *
 *
 */ 
 

typedef struct connect {
	struct gnode	*target;        /* First found target         */
	struct gedge	*edge;          /* and its edge               */
	struct gnode	*target2;       /* Second found target        */
	struct gedge	*edge2;         /* and its edge               */
	struct connect	*internal_next; /* for memory allocation only */
} *CONNECT;

#define	CTARGET(x)	((x)->target)
#define CEDGE(x)	((x)->edge)
#define	CTARGET2(x)	((x)->target2)
#define CEDGE2(x)	((x)->edge2)
#define CINTERN(x)	((x)->internal_next)

/* end. */
