
/*  Edges of a graph
 *  ----------------
 *  During the layout, edges are partitioned into reverted edges,
 *  etc. Thus, we have for `kantenart', or EART, or EKIND, respectively,
 *  the following possibilities:
 *	'U' -> the kind of this edge is undefined, (it is a normal edge)
 *	'S' -> this is a self loop
 *	'D' -> this is a biconnection, i.e. arrows at both sides
 *	'R' -> this is a reverted edge.
 *  If we do not self-layout, we have further:
 *	'l' -> this edge goes to the left
 *	'r' -> this edge goes to the right 
 *
 *  The following tags indicate the initial partitionig, but they are not 
 *  anymore used.
 *	'T' -> this is a tree edge
 *	'F' -> this is a forward edge
 *	'B' -> this is a backward edge (not yet reverted)
 *	'C' -> this is a cross edge
 *
 *  For the x,y co-ordinates, there is a problem if neighoured nodes
 *  are to large. To avoid (1), we allow that edges are bend, see (2) 
 *
 *             -   --------                 -   --------
 *            |A| |        |               |A| |        |
 *             -  |        |                -  |        |
 *              \ |   M    |                |  |   M    |
 *   (1)         \|        |          (2)   |  |        |
 *                \        |                |  |        |
 *                |\       |                |  |        |
 *                 -\------                  \  --------
 *                   \                        \
 *
 *      i.e. each edge have start point and end point, and further a
 *      bend point that has the same x-co-ordinate as the start point
 *      (hold for top_down_layout).
 */

typedef struct gedge {
	struct	gnode	*start;    /* Source node             */
	long	sxloc;             /* and its co-ordinates    */
	long	syloc;
	long	btxloc;		   /* Bend location on top of */
	long	btyloc;		   /* a node                  */
	long	bbxloc;		   /* Bend location on bottom */
	long	bbyloc;
	struct	gnode	*end;      /* Target node             */ 	
	long	exloc;             /* and its co-ordinates    */
	long	eyloc;
	char	orientation; 	   /* updown, northwest, etc. */      
	char	orientation2; 	   /* dito, for double edges  */      

	/* These attributes come directly from the specification */

	char	linestyle;
	char	thickness;
	char	*label;
	int	priority;
	int	horder;
	char	eclass;
	char	color;
	char 	arrowstyle1;	
	char 	arrowstyle2;	
	char	arrowsize1;
	char	arrowcolor1;
	char	arrowsize2;
	char	arrowcolor2;
	char	labelcolor;
	int     anchor;

	/* The layout decides wether an edge is visible or reverted, etc. */

	char	kantenart;	    /* flag 'U', 'S' etc. see above  */
	char	invisible;	    /* 1, if the edge is not visible */

	/* The following two fields are the layout weights ews and ewp of 
	 * the layout algorithm. See the documentation of the layout alg.
         * On drawing, they are further used as number of anchor point.
         */

	long	weights;          /* edge weight succ. (ews)   */
	long	weightp;	  /* edge weight pred. (ewp)   */

	struct	gnode	*labelnode; /* Label node if the edge is replaced */ 	

	/* edgelist links: these are double linked lists */

	struct	gedge	*next;		 /* successor in the edgelist   */
	struct 	gedge	*before; 	 /* predecessor in the edgelist */
	struct	gedge	*internal_next;  /* for memory allocation       */
} *GEDGE;

#define	ESTART(x)	((x)->start)
#define	EEND(x)		((x)->end)
#define	ESTARTX(x)	((x)->sxloc)
#define ESTARTY(x)	((x)->syloc)
#define	ETBENDX(x)	((x)->btxloc)
#define ETBENDY(x)	((x)->btyloc)
#define	EBBENDX(x)	((x)->bbxloc)
#define EBBENDY(x)	((x)->bbyloc)
#define	EENDX(x)	((x)->exloc)
#define	EENDY(x)	((x)->eyloc)
#define EORI(x)		((x)->orientation)
#define EORI2(x)	((x)->orientation2)
#define	ELABEL(x)	((x)->label)
#define	ELABELCOL(x)	((x)->labelcolor)
#define	ELSTYLE(x)	((x)->linestyle)
#define	ETHICKNESS(x)	((x)->thickness)
#define	ECLASS(x)	((x)->eclass)
#define	EPRIO(x)	((x)->priority)
#define	EHORDER(x)	((x)->horder)
#define ECOLOR(x)	((x)->color)
#define	EARROWSTYLE(x)	((x)->arrowstyle1)
#define	EARROWBSTYLE(x)	((x)->arrowstyle2)
#define	EARROWSIZE(x)	((x)->arrowsize1)
#define	EARROWBSIZE(x)	((x)->arrowsize2)
#define	EARROWCOL(x)	((x)->arrowcolor1)
#define	EARROWBCOL(x)	((x)->arrowcolor2)
#define	EANCHOR(x)	((x)->anchor)
#define	ELNODE(x)	((x)->labelnode)
#define	ENEXT(x)	((x)->next)
#define	EBEFORE(x)	((x)->before)
#define EART(x)		((x)->kantenart)
#define	EINVISIBLE(x)	((x)->invisible)
#define	EWEIGHTS(x)	((x)->weights)
#define	EWEIGHTP(x)	((x)->weightp)
#define EINTERN(x)	((x)->internal_next)

/* end. */
