
/*  Graphs             
 *  ======
 *  The complete data structure of a graph is an adjacency list
 *  representation, i.e. all nodes contain adjacency lists of
 *  incoming and outgoing edges.
 *  Furthermore, all nodes contain in NROOT a pointer to the subgraph
 *  summary node they belong to. NROOT of top level nodes is NULL.
 *  Summary nodes of subgraphs contain in NSGRAPH a list of all nodes
 *  that belong to this subgraph.
 *  Furthermore all real nodes are in the node list, while all graph
 *  summary nodes are in the graphlist.
 *  E.g. the top graph contains a node A and a graph S1
 *		which contains nodes B and C and a graph S2
 *				which contains nodes D and E 
 *  NROOT connection:
 *
 *     nodelist --> A --> B ---> C --> D ---> E 
 *                  |      \    /       \    /
 *                 NULL     \  /         \  /NROOT
 *                           \/           \/
 *     graphlist ----------> S1 --------> S2 
 *                            |            |
 *                            V            V
 *                          NULL          NULL
 *
 *                        ,-----------------------------,
 *  NSGRAPH connection:   |            ,-------------,  |
 *                        V            V             |  |
 *                  *->*  *----->*->*  *----->*      |  |
 *                  |  |  |      |  |  |      |      |  |
 *                  V  |  V      V  |  V      V      |  |
 *     nodelist --> A -+> B ---> C -+> D ---> E      |  |
 *                     |            |                |  |
 *                     `-----,      `-----,          |  |
 *                           |            |   NSGRAPH|  |NSGRAPH
 *                           V            V          |  |
 *     graphlist ----------> S1 --------> S2         |  |
 *                           |            |          |  |
 *                           |            `----------'  |
 *                           `--------------------------'
 */


/*  Nodes of a graph
 *  ----------------
 *  Graphs are also represented as nodes, i.e. as summary node.
 *  Graphs s are initially not in the nodelist, but in the graphlist.
 *  In consequence, NINLIST(s) is 0 for them. 
 */

typedef struct gnode {	

	/* Each node has an unique reference number.
 	 * These are used to debug, and to have a stable layout,
	 * because we use the numbers as sorting criteria.
	 */

	long 	refnum;

	/* These attributes come directly from the specification */

	char 	*title;
	char	*label;
	char	*info1;
	char	*info2;
	char	*info3;
	int	width;
	int	height;
	long	sxloc;
	long 	syloc;
	char	textmode;
	char	borderwidth;
	char	folding;
	char	color;
	char	textcolor;
	char	bordercolor;
	char	status;
	char 	shape;
	int	shrink;
	int	stretch;
	int	level;
	int	nhorder;

	/* These attributes are on summary nodes (graphs,regions) */

	struct	gnlist	*subgraph;   /* List of subgraphs       */
	struct	gnode	*root;       /* Root of graph           */
	struct	gnode	*regionrepl; /* Replacement node for    */
				     /* roots of regions        */
	struct	gnlist	*region;     /* List of nodes in region */
	struct 	gnode  	*regroot;    /* Root of region          */

	/* These are the locations used for the layout */

	long	xloc;
	long 	yloc;

	/* nodelist, graphlist links: these are double linked lists */

	struct 	gnode 	*next;	    /* successor in the node/graphlist   */ 
	struct 	gnode	*before;    /* predecessor in the node/graphlist */

	char	in_nodelist;	    /* flag, 1 if node is in nodelist */
	char	invisible;          /* flag, 1 if node is invisible   */

	/* layout attributes: nodes are distributed into layers according
         * to their deeths relatively to the root of the graph, and have
         * positions in these layers.
         */

	char	markiert;           /* marker for depth first search    */    
	char	revert;             /* marker for reverted drawing      */    
	char    anchordummy;	    /* marker for anchor dummy nodes    */
	int	tiefe;		    /* the number (deepth) of the layer */
	int	position;           /* the position in the layer        */
	float	bary;               /* the weight of the bary centering */

	/* The following two fields have sev. purposes: they are the layout
         * weights nws and nwp of the layout algorithm, and on drawing
         * they are used as number of anchor points.
	 * The field weights is also used during the partitioning of
	 * edges as LOWPT of the strongly connected component.
	 * The field weightp is aslo used during the partitioning of
	 * edges as OPENSCC flag of the strongly connected component.
         */

	long	weights;		/* node weight succ. (nws)   */
	long	weightp;		/* node weight pred. (nwp)   */

	/* For partitioning the edges, we need to know cross edges, tree
         * edges, forward edges and backward edges. Thus, we calculate a
         * depth first search (dfs) and protocol by numbers when we entry
         * the dfs for a node and when we leave the dfs of this node.
         */

	long	dfsnum;			/* the dfs entry number      */

	int	indegree;		/* number of incoming edges  */
	int	outdegree;		/* number of outgoing edges  */

	/* To calculate crossings, we need a pointer to the last instance
         * of the upper or lower completion lists. See step2.c
         */

	struct 	dllist	*Vpointer; 	/* this crossing pointer        */

	/* The graph is represented with adjacency lists. We have some
         * fast accesses to the leftest or rightedst prede/successor.
	 * Further, at connections we temporary change the adjacency
	 * lists. Thus we store the original list in save fields.
	 * For crossing calculation, we need to reorder the adjacency
	 * lists. Thus we use a pointer tmpadj to the actual position
	 * in the adjacency list.
         */

	struct	adjedge	*tmpadj;      	/* temporary adjacency list     */
	struct	adjedge	*pred;	       	/* adjacency list: predecessors */
	struct	adjedge	*succ;	       	/* adjacency list: successors   */
	struct	adjedge	*savepred;     	/* adjacency list: predecessors */
	struct	adjedge	*savesucc;     	/* adjacency list: successors   */
	struct	gedge	*predleft;     	/* leftest predecessor          */
	struct	gedge	*predright;    	/* rightest predecessor         */
	struct	gedge	*succleft;	/* leftest successor		*/
	struct	gedge	*succright;	/* rightest successor           */

	struct	connect	*connection;	/* horizontal connection to a   */
					/* neighboured node, see above  */

	/* The next field has two purposes: During parsing, we use it to
         * create a hash table of all nontemporary node, to have fast
	 * access to the title.
	 * Otherwise, it is used for the memory management of temporary 
	 * nodes.
	 */

	struct 	gnode 	*internal_next; 
} *GNODE;

#define	NREFNUM(x)	((x)->refnum)
#define	NTITLE(x)	((x)->title)
#define	NLABEL(x)	((x)->label)
#define	NINFO1(x)	((x)->info1)
#define	NINFO2(x)	((x)->info2)
#define	NINFO3(x)	((x)->info3)
#define NTEXTMODE(x)	((x)->textmode)
#define NWIDTH(x)	((x)->width)
#define	NHEIGHT(x)	((x)->height)
#define	NSTATE(x)	((x)->status)
#define	NSHAPE(x)	((x)->shape)
#define NBORDERW(x)	((x)->borderwidth)
#define	NSX(x)		((x)->sxloc)
#define NSY(x)		((x)->syloc)
#define	NX(x)		((x)->xloc)
#define NY(x)		((x)->yloc)
#define	NFOLDING(x)	((x)->folding)
#define NCOLOR(x)	((x)->color)
#define	NTCOLOR(x)	((x)->textcolor)
#define	NBCOLOR(x)	((x)->bordercolor)
#define	NSHRINK(x)	((x)->shrink)
#define	NSTRETCH(x)	((x)->stretch)
#define	NLEVEL(x)	((x)->level)
#define	NHORDER(x)	((x)->nhorder)
#define	NSGRAPH(x)	((x)->subgraph)
#define	NROOT(x)	((x)->root)
#define NREGREPL(x)	((x)->regionrepl)
#define NREGION(x)	((x)->region)
#define	NREGROOT(x)	((x)->regroot)
#define	NNEXT(x)	((x)->next)
#define NBEFORE(x)	((x)->before)
#define NINLIST(x)	((x)->in_nodelist)
#define	NINVISIBLE(x)	((x)->invisible)
#define	NTIEFE(x)	((x)->tiefe)
#define	NPOS(x)		((x)->position)
#define	NMARK(x)	((x)->markiert)
#define	NREVERT(x)	((x)->revert)
#define	NANCHORNODE(x)	((x)->anchordummy)
#define	NBARY(x)	((x)->bary)
#define	NLOWPT(x)	((x)->weights)
#define	NOPENSCC(x)	((x)->weightp)
#define	NWEIGHTS(x)	((x)->weights)
#define	NWEIGHTP(x)	((x)->weightp)
#define	NDFS(x)		((x)->dfsnum)
#define	NHIGHPRIO(x)	((x)->dfsnum)
#define	NINDEG(x)	((x)->indegree)
#define	NOUTDEG(x)	((x)->outdegree)
#define	NVPTR(x)	((x)->Vpointer)
#define	NTMPADJ(x)	((x)->tmpadj)
#define	NPRED(x)	((x)->pred)
#define NSUCC(x)	((x)->succ)
#define	NSVPRED(x)	((x)->savepred)
#define NSVSUCC(x)	((x)->savesucc)
#define	NPREDL(x)	((x)->predleft)
#define	NPREDR(x)	((x)->predright)
#define	NSUCCL(x)	((x)->succleft)
#define	NSUCCR(x)	((x)->succright)
#define NCONNECT(x)	((x)->connection)
#define NINTERN(x)	((x)->internal_next)

#define	NCONTAR(x)	(CTARGET(NCONNECT(x)))
#define NCONEDG(x)	(CEDGE(NCONNECT(x)))
#define	NCONTAR2(x)	(CTARGET2(NCONNECT(x)))
#define NCONEDG2(x)	(CEDGE2(NCONNECT(x)))

/* end. */
